import initAjax from '../lib/index'
import { getType } from '../lib/utils'
import type { RequestMethods } from '../lib/types.d'
import type { OldAjaxMethods } from '../lib/compatibleMethods.d'

// 生成随机URL，避免测试用例之间的干扰
function getRandomUrl(len = 4) {
  const aCode = 'a'.charCodeAt(0)
  const getIdx = () => Math.floor(Math.random() * 26) + aCode
  return new Array(len).fill(null)
    .map(() => `${String.fromCharCode(getIdx())}${String.fromCharCode(getIdx())}`)
    .join('/')
}

function getRandomKey(len = 4) {
  const aCode = 'a'.charCodeAt(0)
  const getIdx = () => Math.floor(Math.random() * 26) + aCode
  return new Array(len).fill(null)
    .map(() => String.fromCharCode(getIdx()))
    .join('')
}

jest.mock('axios')

const originAbortController = global.AbortController
function undoAbortController() {
  // 微信环境没有AbortController
  Object.defineProperty(global, 'AbortController', {
    value: undefined
  })
}

function restoreAbortController() {
  Object.defineProperty(global, 'AbortController', {
    value: originAbortController
  })
}

const methods: RequestMethods[] = ['get', 'post', 'head', 'patch', 'put', 'delete']
const hisMethods: OldAjaxMethods[] = ['create', 'deletes', 'query', 'patchWay', 'putWay']

const methodsStr = methods.join('|')
const hisMethodsStr = hisMethods.join('|')

describe('initAjax test', () => {

  test('初始化测试', () => {
    const ajax = initAjax({})
    expect(ajax.$options).toEqual({});
    expect(ajax.$engine).toBeDefined();
    expect(ajax.canceller).toBeDefined();
    // request, interceptors
    expect(ajax.request).not.toBeUndefined()
    expect(ajax.interceptors).not.toBeUndefined()
  })

  test.each(methods)('ajax方法检测', (method) => {
    const ajax = initAjax({})
    expect(typeof ajax[method]).toBe('function')
  })

  test.each(hisMethods)('ajax兼容方法检测', (method) => {
    const ajax = initAjax({})
    expect(typeof ajax[method]).toBe('function')
  })

  test.each([...methods, ...hisMethods])('ajax[method]调用应该返回函数', (method) => {
    const ajax = initAjax({})
    expect(typeof ajax[method](getRandomUrl(), {})).toBe('function')
  })

  test.each([...methods, ...hisMethods])('ajax[method]()返回的函数再调用应该返回Promise对象', (method) => {
    const ajax = initAjax({})
    expect(getType(ajax[method](getRandomUrl(), {})())).toBe('promise')
  })

  test('ajax.canceller的任意属性均为函数', () => {

    const ajax = initAjax({})

    const randomKey = getRandomKey()

    expect(typeof ajax.canceller[randomKey]).toBe('function')
    const fn = ajax.canceller[randomKey]
    // 没有生成请求，获取任意属性，会给出提示
    expect(fn()).toMatch(`ajax.canceller对象上不存在名为${randomKey}的函数！`)
  })
})

describe('手动取消请求测试', () => {

  describe('web或node环境', () => {

    test.each(methods)(methodsStr, async (method) => {
  
      const ajax = initAjax({})
  
      // 发出请求
      const cancelKey = getRandomKey()
      const fetch = ajax[method](getRandomUrl())({}, {cancelKey})
      // 取消请求
      ajax.canceller[cancelKey]()
  
      // 模拟的axios直接返回了请求被取消，实际使用可能不是这样
      await expect(fetch).rejects.toMatch('请求被取消')
    })

  })

  describe('wx环境', () => {
    beforeAll(() => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              data: {}
            }, 50)
          })
  
          return { abort: jest.fn() }
        })
      }
      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })
    
    test.each(methods)(methodsStr, async (method) => {
  
      const ajax = initAjax({})
  
      // 发出请求
      const cancelKey = getRandomKey()
      const fetch = ajax[method](getRandomUrl())({}, {cancelKey})
      // 取消请求
      ajax.canceller[cancelKey]()
  
      await expect(fetch).rejects.toHaveProperty('message', '请求被取消！')
      await expect(fetch).rejects.toHaveProperty('code', 'ERR_CANCELED')
    })
  })

})

describe('多个具有相同cancelKey，但不同参数的请求发出并取消测试', () => {

  describe('web或node环境', () => {
    
    test.each(methods)(methodsStr, async (method) => {
  
      const ajax = initAjax({})
  
      // 发出请求
      const cancelKey = getRandomKey()
      // 请求参数不同
      const fetchA = ajax[method](getRandomUrl())({a:1}, {cancelKey})
      const fetchB = ajax[method](getRandomUrl())({a:2}, {cancelKey})
      const fetchC = ajax[method](getRandomUrl())({a:3}, {cancelKey})
      // 取消请求
      ajax.canceller[cancelKey]()
  
      // 请求都应该被取消
      await expect(fetchA).rejects.toMatch('请求被取消')
      await expect(fetchB).rejects.toMatch('请求被取消')
      await expect(fetchC).rejects.toMatch('请求被取消')
    })

  })

  describe('wx环境', () => {
    beforeAll(() => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              data: {}
            }, 50)
          })
  
          return { abort: jest.fn() }
        })
      }

      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })

    test.each(methods)(methodsStr, async (method) => {
  
      const ajax = initAjax({})
  
      // 发出请求
      const cancelKey = getRandomKey()
      const fetchA = ajax[method](getRandomUrl())({a:1}, {cancelKey})
      const fetchB = ajax[method](getRandomUrl())({a:2}, {cancelKey})
      const fetchC = ajax[method](getRandomUrl())({a:3}, {cancelKey})
      // 取消请求
      ajax.canceller[cancelKey]()
  
      await expect(fetchA).rejects.toHaveProperty('message', '请求被取消！')
      await expect(fetchB).rejects.toHaveProperty('message', '请求被取消！')
      await expect(fetchC).rejects.toHaveProperty('message', '请求被取消！')
    })
  })
})

describe('多个具有相同cancelKey，不同参数的请求，交替发出并取消测试', () => {

  describe('web或node环境', () => {

    test.each(methods)(methodsStr, async (method) => {
  
      const ajax = initAjax({})
  
      // 发出请求
      const cancelKey = getRandomKey()
      // 请求参数不同，且交替发出取消请求，最后一个没有被取消的请求应该正常返回
      const fetchA = ajax[method](getRandomUrl())({a:1}, {cancelKey})
      ajax.canceller[cancelKey]()
      const fetchB = ajax[method](getRandomUrl())({a:2}, {cancelKey})
      ajax.canceller[cancelKey]()
      const fetchC = ajax[method](getRandomUrl())({a:3}, {cancelKey})
      // 取消请求
  
      // 请求都应该被取消
      await expect(fetchA).rejects.toMatch('请求被取消')
      await expect(fetchB).rejects.toMatch('请求被取消')
      await expect(fetchC).resolves.not.toBeNaN()
    })

  })

  describe('wx环境', () => {
    beforeAll(() => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              data: {}
            }, 50)
          })
  
          return { abort: jest.fn() }
        })
      }
      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })

    test.each(methods)(methodsStr, async (method) => {
  
      const ajax = initAjax({})
  
      // 发出请求
      const cancelKey = getRandomKey()
      const fetchA = ajax[method](getRandomUrl())({a:1}, {cancelKey})
      ajax.canceller[cancelKey]()
      const fetchB = ajax[method](getRandomUrl())({a:2}, {cancelKey})
      ajax.canceller[cancelKey]()
      const fetchC = ajax[method](getRandomUrl())({a:3}, {cancelKey})
      // 取消请求
  
      await expect(fetchA).rejects.toHaveProperty('message', '请求被取消！')
      await expect(fetchB).rejects.toHaveProperty('message', '请求被取消！')
      await expect(fetchC).resolves.toHaveProperty('data', {})
    })

  })

})

describe('300ms内的并行触发的重复请求不会触发二次网络请求，后一个请求复用前一个结果', () => {

  describe('web或node环境', () => {

    test.each(methods)(methodsStr, async (method) => {
      const ajax = initAjax({reUseDuration: 300})
      const url = getRandomUrl()
      // 并行发出请求
      const req1 = ajax[method](url)()
      const req2 = ajax[method](url)()
      const res1 = await req1
      const res2 = await req2
  
      // mock的axios.request每次会返回一个随机数，如果复用上一个请求，两者应该相等
      expect(res1).toBe(res2)
    })
  
    test.each(hisMethods)(hisMethodsStr, async (method) => {
      const ajax = initAjax({reUseDuration: 300})
      const url = getRandomUrl()
      // 并行发出请求，旧的方法配置只能在ajax[method]的第二个参数写
      const req1 = ajax[method](url)()
      const req2 = ajax[method](url)()
      const res1 = await req1
      const res2 = await req2
  
      // mock的axios.request每次会返回一个随机数，如果复用上一个请求，两者应该相等
      expect(res1).toBe(res2)
    })

  })

  describe('wx环境', () => {

    beforeAll(() => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              data: {a: 1}
            }, 50)
          })
        })
      }

      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })

    test.each(methods)(methodsStr,async (method: RequestMethods) => {

      const ajax = initAjax({}) // reUseDuration默认值为300
      const url = getRandomUrl()
      // 并行发出请求
      const req1 = ajax[method](url)()
      const req2 = ajax[method](url)()
      const res1 = await req1
      const res2 = await req2
  
      expect(res1).toBe(res2)
    })
  
    test.each(hisMethods)(hisMethodsStr, async (method) => {
      const ajax = initAjax({})
      // 验证微信环境，web和node下为undefined
      expect(ajax.$options.adapter).not.toBeUndefined()
  
      const url = getRandomUrl()
      // 并行发出请求，旧的方法配置只能在ajax[method]的第二个参数写
      const req1 = ajax[method](url)()
      const req2 = ajax[method](url)()
      const res1 = await req1
      const res2 = await req2
  
      // mock的axios.request每次会返回一个随机数，如果复用上一个请求，两者应该相等
      expect(res1).toBe(res2)
    })
  })

})

describe('reUseDuration配置时间小于请求返回的时间，即使前一个请求没有返回，但在reUseDuration之后发出的请求不会复用前一个结果', () => {

  describe('web或node环境', () => {

    test.each(methods)(methodsStr, (method, done) => {
      const ajax = initAjax({reUseDuration: 30}) // mock的axios返回时间为50ms
      const url = getRandomUrl()
  
      const req1 = ajax[method](url)()
      setTimeout(async () => {
        // 第二个请求在reUseDuration之后，请求返回之前发出
        const req2 = ajax[method](url)()
        const res1 = await req1
        const res2 = await req2
        // mock的axios.request每次会返回一个随机数，两个请求均返回，但结果并不相同
        expect(res1).not.toBeNaN()
        expect(res2).not.toBeNaN()
        expect(res1).not.toBe(res2)
        done()
      }, 40)
    })

    test.each(hisMethods)(hisMethodsStr, (method, done) => {
      const ajax = initAjax({reUseDuration: 30})
      const url = getRandomUrl()
      
      const req1 = ajax[method](url)()
      setTimeout(async () => {
        // 第二个请求在reUseDuration之后，请求返回之前发出
        const req2 = ajax[method](url)()
        const res1 = await req1
        const res2 = await req2
    
        // mock的axios.request每次会返回一个随机数，两个请求均返回，但结果并不相同
        expect(res1).not.toBeNaN()
        expect(res2).not.toBeNaN()
        expect(res1).not.toBe(res2)
        done()
      }, 40)
    })

  })

  describe('wx环境', () => {

    beforeAll(() => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              data: Math.random()
            }, 50)
          })
        })
      }
  
      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })
    
    test.each(methods)(methodsStr, (method: RequestMethods, done) => {
  
      const ajax = initAjax({reUseDuration: 30})
      const url = getRandomUrl()

      expect(ajax.$options.adapter).not.toBeUndefined() // 证明是微信环境
      
      const req1 = ajax[method](url)()
      setTimeout(async () => {
        // 第二个请求在reUseDuration之后，请求返回之前发出
        const req2 = ajax[method](url)()
        const res1 = await req1
        const res2 = await req2
    
        expect(res1.data).not.toBeNaN()
        expect(res2.data).not.toBeNaN()
        console.log('res2:', res2, res1)
        expect(res1.data).not.toEqual(res2.data)
        done()
      }, 40)
    })

    test.each(hisMethods)(hisMethodsStr, (method, done) => {

      const ajax = initAjax({reUseDuration: 30})
      // 验证微信环境，web和node下为undefined
      expect(ajax.$options.adapter).not.toBeUndefined()
  
      const url = getRandomUrl()
      
      const req1 = ajax[method](url)()
      setTimeout(async () => {
        // 第二个请求在reUseDuration之后，请求返回之前发出
        const req2 = ajax[method](url)()
        const res1 = await req1
        const res2 = await req2
    
        // 返回的对象虽然相同，但是并不是同一个对象，证明不是复用
        expect(res1.data).not.toBeNaN()
        expect(res2.data).not.toBeNaN()
        expect(res1.data).not.toEqual(res2.data)
        done()
      }, 40)
    })
  })

})

describe('触发间隔小于300ms，但第二个请求是在第一个请求返回后触发，依然会触发二次网络请求', () => {

  describe('web或node环境', () => {

    test.each(methods)(methodsStr, async (method) => {
      const ajax = initAjax({})
  
      const url = getRandomUrl()
      // 发出请求，50ms返回结果，见__mocks__/axios.request
      const res1 = await ajax[method](url)()
      const res2 = await ajax[method](url)()
  
      // mock的axios.request每次会返回一个随机数
      expect(res1).not.toBe(res2)
      expect(res1).not.toBeNaN()
      expect(res2).not.toBeNaN()
    })

    test.each(hisMethods)(hisMethodsStr, async (method) => {
      const ajax = initAjax({})
      const url = getRandomUrl()
  
      // 发出请求，50ms返回结果，见__mocks__/axios.request
      const res1 = await ajax[method](url)()
      const res2 = await ajax[method](url)()
  
      // mock的axios.request每次会返回一个随机数
      expect(res1).not.toBe(res2)
      expect(res1).not.toBeNaN()
      expect(res2).not.toBeNaN()
    })

  })

  describe('wx环境', () => {
    beforeAll(() => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              data: {a: 1}
            }, 50)
          })
        })
      }
      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })

    test.each(methods)(methodsStr, async (method) => {
      const ajax = initAjax({})
  
      const url = getRandomUrl()
      // 发出请求，50ms返回结果，见__mocks__/axios.request
      const res1 = await ajax[method](url)()
      const res2 = await ajax[method](url)()
  
      expect(res1).not.toBe(res2)
      expect(res1.data).toEqual({a:1})
      expect(res2.data).toEqual({a:1})
      expect(res1.config.url).toBe(url)
      expect(res2.config.url).toBe(url)
  
    })
  
    test.each(hisMethods)(hisMethodsStr, async (method) => {
  
      const ajax = initAjax({})
      const url = getRandomUrl()
  
      // 发出请求，50ms返回结果，见__mocks__/axios.request
      const res1 = await ajax[method](url)()
      const res2 = await ajax[method](url)()
  
      expect(res1).not.toBe(res2)
      expect(res1.data).toEqual({a:1})
      expect(res2.data).toEqual({a:1})
      expect(res1.config.url).toBe(url)
      expect(res2.config.url).toBe(url)
    })
  })

})

describe('validateStatus配置测试', () => {
    
  describe('web或node环境', () => {
    // TODO：待补充，需要在__mocks__/axios中模拟validateStatus功能
  })

  describe('wx环境', () => {
    beforeAll(() => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              // 微信返回400
              statusCode: 400,
              data: {}
            }, 50)
          })
        })
      }

      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })

    test.each(methods)(methodsStr, async (method) => {
      
      const ajax = initAjax({
        validateStatus: (status) => {
          // 小于200，或大于300，判为请求失败
          if (status < 200 || status > 300) return false
          return true
        }
      })
      const fetch = ajax[method](getRandomUrl())
  
      await expect(fetch()).rejects.toHaveProperty('message', 'Request failed with status code 400')
    })

    test.each(hisMethods)(hisMethodsStr, async (method) => {
  
      const ajax = initAjax({
        validateStatus: (status) => {
          // 小于200，或大于300，判为请求失败
          if (status < 200 || status > 300) return false
          return true
        }
      })
      const fetch = ajax[method](getRandomUrl())
  
      await expect(fetch()).rejects.toHaveProperty('message', 'Request failed with status code 400')
    })
  })
})

describe('发送请求异常测试', () => {

  describe('web或node环境', () => {
    // TODO
  })

  describe('wx环境', () => {
    beforeAll(() => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.fail({
              errMsg: '网络连接错误',
              errno: 1101
            }, 50)
          })
        })
      }

      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })

    test.each([...methods, ...hisMethods])(`${methodsStr}|${hisMethodsStr}`, async (method) => {
      
      const ajax = initAjax({})
  
      const fetch = ajax[method](getRandomUrl())
  
      await expect(fetch()).rejects.toHaveProperty('message', '网络连接错误')
      await expect(fetch()).rejects.toHaveProperty('code', 1101)
      await expect(fetch()).rejects.toHaveProperty('config')
      await expect(fetch()).rejects.toHaveProperty('request')
      await expect(fetch()).rejects.toHaveProperty('response', undefined)
    })
  })

})

describe('cancelLastSameUrl为true，第二个相同URL的get请求，会将第一个get请求cancel掉', () => {

  describe('web和node环境', () => {

    test('两个请求', async () => {
      const ajax = initAjax({})
      const url = getRandomUrl()
      // 发出请求，50ms返回结果，见__mocks__/axios.request
      // 两个请求URL和方法相同，第一个会取消第二个请求
      const res1 = ajax.get(url, {})({a:1}, {cancelLastSameUrl: true})
      const res2 = ajax.get(url, {})({}, {cancelLastSameUrl: true})

      // mock的axios.request每次会返回一个随机数
      await expect(res1).rejects.toMatch('请求被取消')
      await expect(res2).resolves.not.toBeNaN()
    })

    test('连续三个请求', async () => {
      const ajax = initAjax({})
      const url = getRandomUrl()
      // 发出请求，50ms返回结果，见__mocks__/axios.request
      // 两个请求URL和方法相同，第一个会取消第二个请求
      const res1 = ajax.get(url, {})({a: 1}, {cancelLastSameUrl: true})
      const res2 = ajax.get(url, {})({a: 2}, {cancelLastSameUrl: true})
      const res3 = ajax.get(url, {})({a: 3}, {cancelLastSameUrl: true})

      // mock的axios.request每次会返回一个随机数
      await expect(res1).rejects.toMatch('请求被取消')
      await expect(res2).rejects.toMatch('请求被取消')
      await expect(res3).resolves.not.toBeNaN()
    })

    test('使用cancelLastSameUrl，但如果两个请求完全相同，且同时发出，第二个请求返回第一个请求的请求结果，不会取消第一个请求', async () => {
      const ajax = initAjax({})
      const url = getRandomUrl()

      // 发出请求，50ms返回结果，见__mocks__/axios.request
      const req1 = ajax.get(url, {})({}, {cancelLastSameUrl: true})
      const req2 = ajax.get(url, {})({}, {cancelLastSameUrl: true})

      const res1 = await req1
      const res2 = await req2
      // mock的axios.request每次会返回一个随机数
      expect(res1).toBe(res2)
    })
  })

  describe('wx环境', () => {

    beforeAll(() => {
      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })

    test('两个请求', async () => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              data: {}
            }, 50)
          })

          return { abort: jest.fn() }
        })
      }

      const ajax = initAjax({})
      const url = getRandomUrl()

      // 验证微信环境
      expect(ajax.$options.adapter).not.toBeUndefined()
      expect(typeof ajax.$options.adapter).toBe('function')
      // 发出请求，50ms返回结果，见__mocks__/axios.request
      // 两个请求URL和方法相同，第一个会取消第二个请求
      const res1 = ajax.get(url, {})({a:1}, {cancelLastSameUrl: true})
      const res2 = ajax.get(url, {})({}, {cancelLastSameUrl: true})

      // mock的axios.request每次会返回一个随机数
      await expect(res1).rejects.toHaveProperty('code', 'ERR_CANCELED')
      await expect(res1).rejects.toHaveProperty('message', '请求被取消！')
      await expect(res1).rejects.toHaveProperty('config')
      await expect(res1).rejects.toHaveProperty('request')
      await expect(res1).rejects.toHaveProperty('response')
      // 第二个请求返回结果
      await expect(res2).resolves.toHaveProperty('data', {})
    })
  
    test('三个请求', async () => {
      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              data: {}
            }, 100)
          })

          return { abort: jest.fn() }
        })
      }

      const ajax = initAjax({})
      const url = getRandomUrl()

      // 验证微信环境
      expect(ajax.$options.adapter).not.toBeUndefined()
      expect(typeof ajax.$options.adapter).toBe('function')
      // 发出请求，50ms返回结果，见__mocks__/axios.request
      // 两个请求URL和方法相同，第一个会取消第二个请求
      const res1 = ajax.get(url, {})({a: 1}, {cancelLastSameUrl: true})
      const res2 = ajax.get(url, {})({a: 2}, {cancelLastSameUrl: true})
      const res3 = ajax.get(url, {})({a: 3}, {cancelLastSameUrl: true})

      // mock的axios.request每次会返回一个随机数
      await expect(res1).rejects.toHaveProperty('code', 'ERR_CANCELED')
      await expect(res1).rejects.toHaveProperty('message', '请求被取消！')
      await expect(res2).rejects.toHaveProperty('code', 'ERR_CANCELED')
      await expect(res2).rejects.toHaveProperty('message', '请求被取消！')

      // 第二个请求返回结果
      await expect(res3).resolves.toHaveProperty('data', {})
    })

    test('使用cancelLastSameUrl，但如果两个请求完全相同，且同时发出，第二个请求返回第一个请求的请求结果，不会取消第一个请求', async () => {

      global.wx = {
        request: jest.fn((config: any) => {
          setTimeout(() => {
            config.success({
              data: {}
            }, 50)
          })

          return { abort: jest.fn() }
        })
      }
      const ajax = initAjax({})
      const url = getRandomUrl()

      // 发出请求，50ms返回结果，见__mocks__/axios.request
      const req1 = ajax.get(url, {})({}, {cancelLastSameUrl: true})
      const req2 = ajax.get(url, {})({}, {cancelLastSameUrl: true})

      const res1 = await req1
      const res2 = await req2
      // mock的axios.request每次会返回一个随机数
      expect(res1).toBe(res2)
    })
  })
})

describe('mock配置测试', () => {
  describe('web或node环境', () => {
    test.each(methods)('基本功能', async (method) => {
      const ajax = initAjax({})
      const url = getRandomUrl()

      const res1 = ajax[method](url, {})({a:1}, {mock: {mock:true}})
      await expect(res1).resolves.toEqual({mock:true})
    })
    // 历史方法没有兼容该功能
    // test.each(hisMethods)('基本功能(废弃的方法)', async (method) => {
    //   const ajax = initAjax({})
    //   const url = 'web/get/mocked/res/duplicate'

    //   const res1 = ajax[method](url, {mock: {mock:true}})({a:1})
    //   await expect(res1).resolves.toEqual({mock:true})
    // })
  })

  describe('wx环境', () => {

    beforeAll(() => {
      global.wx = {
        request: jest.fn()
      }
      undoAbortController()
    })

    afterAll(() => {
      delete global.wx
      restoreAbortController()
    })

    test.each(methods)('基本功能', async (method) => {

      const ajax = initAjax({})
      const url = getRandomUrl()

      expect(ajax.$options.adapter).not.toBeUndefined()
      const res1 = ajax[method](url, {})({a:1}, {mock: {mock:true}})
      await expect(res1).resolves.toEqual({mock:true})
    })
    // 历史方法没有兼容该功能
    // test.each(hisMethods)('基本功能(废弃方法)', async (method) => {
    //   const ajax = initAjax({})
    //   const url = 'wx/get/mocked/res/duplicate'

    //   expect(ajax.$options.adapter).not.toBeUndefined()
    //   // 即将废弃的方法，配置只能在ajax[method]()的第二个参数
    //   const res1 = ajax[method](url, {mock: {mock:true}})({a:1})
    //   await expect(res1).resolves.toEqual({mock:true})
    // })
  })
})
